@@ -4,8 +4,9 @@
  * Generate a printable version of the group database
  */
 /*
- * Arnold Robbins, arnold@gnu.org, May 1993
+ * Arnold Robbins, arnold@skeeve.com, May 1993
  * Public Domain
+ * December 2010, move to ANSI C definition for main().
  */
 
 /* For OS/2, do nothing. */
@@ -24,16 +25,18 @@ int main() { return 0; }
 #include <grp.h>
 
 int
-main(argc, argv)
-int argc;
-char **argv;
+main(int argc, char **argv)
 {
     struct group *g;
     int i;
 
     while ((g = getgrent()) != NULL) {
+#ifdef ZOS_USS
+        printf("%s:%ld:", g->gr_name, (long) g->gr_gid);
+#else
         printf("%s:%s:%ld:", g->gr_name, g->gr_passwd,
                                      (long) g->gr_gid);
+#endif
         for (i = 0; g->gr_mem[i] != NULL; i++) {
             printf("%s", g->gr_mem[i]);
             if (g->gr_mem[i+1] != NULL)
