@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2013 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2014 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -24,7 +24,7 @@
  */
 
 /* FIX THIS BEFORE EVERY RELEASE: */
-#define UPDATE_YEAR	2013
+#define UPDATE_YEAR	2014
 
 #include "awk.h"
 #include "getopt.h"
@@ -133,7 +133,7 @@ static bool disallow_var_assigns = false;	/* true for --exec */
 static void add_preassign(enum assign_type type, char *val);
 
 int do_flags = false;
-bool do_optimize = true;		/* apply default optimizations */
+bool do_optimize = false;		/* apply default optimizations */
 static int do_nostalgia = false;	/* provide a blast from the past */
 static int do_binary = false;		/* hands off my data! */
 static int do_version = false;		/* print version info */
@@ -157,42 +157,43 @@ GETGROUPS_T *groupset;		/* current group set */
 int ngroups;			/* size of said set */
 #endif
 
-void (*lintfunc)(const char *mesg, ...) = warning;
+void (*lintfunc)(const char *mesg, ...) = r_warning;
 
+/* Sorted by long option name! */
 static const struct option optab[] = {
-	{ "traditional",	no_argument,		NULL,	'c' },
-	{ "lint",		optional_argument,	NULL,	'L' },
-	{ "lint-old",		no_argument,		NULL,	't' },
-	{ "optimize",		no_argument,		NULL,	'O' },
-	{ "posix",		no_argument,		NULL,	'P' },
-	{ "nostalgia",		no_argument,		& do_nostalgia,	1 },
-	{ "gen-pot",		no_argument,		NULL,	'g' },
-	{ "non-decimal-data",	no_argument,		NULL,	'n' },
-	{ "pretty-print",	optional_argument,	NULL,	'o' },
-	{ "profile",		optional_argument,	NULL,	'p' },
-	{ "debug",		optional_argument,	NULL,	'D' },
+	{ "assign",		required_argument,	NULL,	'v' },
+	{ "bignum",		no_argument,		NULL,	'M' },
+	{ "characters-as-bytes", no_argument,		& do_binary,	 'b' },
 	{ "copyright",		no_argument,		NULL,	'C' },
+	{ "debug",		optional_argument,	NULL,	'D' },
+	{ "dump-variables",	optional_argument,	NULL,	'd' },
+	{ "exec",		required_argument,	NULL,	'E' },
 	{ "field-separator",	required_argument,	NULL,	'F' },
 	{ "file",		required_argument,	NULL,	'f' },
-	{ "re-interval",	no_argument,		NULL,	'r' },
-	{ "source",		required_argument,	NULL,	'e' },
-	{ "load",		required_argument,	NULL,	'l' },
-	{ "dump-variables",	optional_argument,	NULL,	'd' },
-	{ "assign",		required_argument,	NULL,	'v' },
-	{ "version",		no_argument,		& do_version, 'V' },
+	{ "gen-pot",		no_argument,		NULL,	'g' },
 	{ "help",		no_argument,		NULL,	'h' },
-	{ "exec",		required_argument,	NULL,	'E' },
-	{ "use-lc-numeric",	no_argument,		& use_lc_numeric, 1 },
-	{ "characters-as-bytes", no_argument,		& do_binary,	 'b' },
-	{ "sandbox",		no_argument,		NULL, 	'S' },
-	{ "bignum",		no_argument,		NULL,	'M' },
+	{ "include",		required_argument,	NULL,	'i' },
+	{ "lint",		optional_argument,	NULL,	'L' },
+	{ "lint-old",		no_argument,		NULL,	't' },
+	{ "load",		required_argument,	NULL,	'l' },
+	{ "non-decimal-data",	no_argument,		NULL,	'n' },
+	{ "nostalgia",		no_argument,		& do_nostalgia,	1 },
+	{ "optimize",		no_argument,		NULL,	'O' },
 #if defined(YYDEBUG) || defined(GAWKDEBUG)
-	{ "parsedebug",		no_argument,		NULL,		'Y' },
+	{ "parsedebug",		no_argument,		NULL,	'Y' },
 #endif
+	{ "posix",		no_argument,		NULL,	'P' },
+	{ "pretty-print",	optional_argument,	NULL,	'o' },
+	{ "profile",		optional_argument,	NULL,	'p' },
+	{ "re-interval",	no_argument,		NULL,	'r' },
+	{ "sandbox",		no_argument,		NULL, 	'S' },
+	{ "source",		required_argument,	NULL,	'e' },
+	{ "traditional",	no_argument,		NULL,	'c' },
+	{ "use-lc-numeric",	no_argument,		& use_lc_numeric, 1 },
+	{ "version",		no_argument,		& do_version, 'V' },
 	{ NULL, 0, NULL, '\0' }
 };
 
-
 /* main --- process args, parse program, run it, clean up */
 
 int
@@ -284,6 +285,22 @@ main(int argc, char **argv)
 #ifdef SIGBUS
 	(void) signal(SIGBUS, catchsig);
 #endif
+#ifdef SIGPIPE
+	/*
+	 * Ignore SIGPIPE so that writes to pipes that fail don't
+	 * kill the process but instead return -1 and set errno.
+	 * That lets us print a fatal message instead of dieing suddenly.
+	 *
+	 * Note that this requires ignoring EPIPE when writing and
+	 * flushing stdout/stderr in other parts of the program. E.g.,
+	 *
+	 * 	gawk 'BEGIN { print "hi" }' | exit
+	 *
+	 * should not give us "broken pipe" messages --- mainly because
+	 * it did not do so in the past and people would complain.
+	 */
+	signal(SIGPIPE, SIG_IGN);
+#endif
 
 	(void) sigsegv_install_handler(catchsegv);
 #define STACK_SIZE (16*1024)
@@ -438,7 +455,7 @@ main(int argc, char **argv)
 			break;
 
 		case 'O':
-			do_optimize++;
+			do_optimize = true;
 			break;
 
 		case 'p':
@@ -722,6 +739,9 @@ out:
 
 	if (do_debug)
 		debug_prog(code_block);
+	else if (do_pretty_print && ! do_debug && getenv("GAWK_NO_PP_RUN") != NULL)
+		/* hack to run pretty printer only. need a better solution */
+		;
 	else
 		interpret(code_block);
 
@@ -838,8 +858,13 @@ By default it reads standard input and writes standard output.\n\n"), fp);
 	fflush(fp);
 
 	if (ferror(fp)) {
-		if (fp == stdout)
-			warning(_("error writing standard output (%s)"), strerror(errno));
+		/* don't warn about stdout/stderr if EPIPE, but do error exit */
+		if (errno != EPIPE) {
+			if (fp == stdout)
+				warning(_("error writing standard output (%s)"), strerror(errno));
+			else if (fp == stderr)
+				warning(_("error writing standard error (%s)"), strerror(errno));
+		}
 		exit(EXIT_FAILURE);
 	}
 
@@ -876,7 +901,9 @@ along with this program. If not, see http://www.gnu.org/licenses/.\n");
 	fflush(stdout);
 
 	if (ferror(stdout)) {
-		warning(_("error writing standard output (%s)"), strerror(errno));
+		/* don't warn about stdout if EPIPE, but do error exit */
+		if (errno != EPIPE)
+			warning(_("error writing standard output (%s)"), strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
